@@ -8,32 +8,31 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *                                                                              
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *                                                                              
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ * MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include "xfae-window.h"
 #include "xfae-dialog.h"
 #include "xfae-model.h"
-#include "xfae-window.h"
 
+#include <libxfce4ui/libxfce4ui.h>
 
-
-static void     xfae_window_class_init          (XfaeWindowClass  *klass);
-static void     xfae_window_init                (XfaeWindow       *window);
 static void     xfae_window_add                 (XfaeWindow       *window);
 static void     xfae_window_remove              (XfaeWindow       *window);
+static void     xfae_window_edit                (XfaeWindow       *window);
 static gboolean xfae_window_button_press_event  (GtkWidget        *treeview,
                                                  GdkEventButton   *event,
                                                  XfaeWindow       *window);
@@ -95,6 +94,7 @@ xfae_window_init (XfaeWindow *window)
 
   img = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
   gtk_box_pack_start (GTK_BOX (hbox), img, FALSE, FALSE, 0);
+  gtk_misc_set_alignment (GTK_MISC (img), 0.50, 0.00);
   gtk_widget_show (img);
 
   label = g_object_new (GTK_TYPE_LABEL,
@@ -102,7 +102,9 @@ xfae_window_init (XfaeWindow *window)
                         "label", _("Below is the list of applications that will be started "
                                    "automatically when you login to your Xfce desktop, "
                                    "in addition to the applications that were saved when "
-                                   "you logged out last time:"),
+                                   "you logged out last time. Cursive applications belong "
+                                   "to another desktop environment, but you can still enable "
+                                   "them if you want."),
                         "xalign", 0.0f,
                         NULL);
   gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
@@ -114,7 +116,7 @@ xfae_window_init (XfaeWindow *window)
                        "vadjustment", NULL,
                        "shadow-type", GTK_SHADOW_IN,
                        "vscrollbar-policy", GTK_POLICY_AUTOMATIC,
-                       "hscrollbar-policy", GTK_POLICY_NEVER,
+                       "hscrollbar-policy", GTK_POLICY_AUTOMATIC,
                        NULL);
   gtk_box_pack_start (GTK_BOX (vbox), swin, TRUE, TRUE, 0);
   gtk_widget_show (swin);
@@ -131,6 +133,7 @@ xfae_window_init (XfaeWindow *window)
 
   model = xfae_model_new ();
   gtk_tree_view_set_model (GTK_TREE_VIEW (window->treeview), model);
+  gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (window->treeview), XFAE_MODEL_COLUMN_TOOLTIP);
   g_object_unref (G_OBJECT (model));
 
   window->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->treeview));
@@ -161,9 +164,9 @@ xfae_window_init (XfaeWindow *window)
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
   gtk_tree_view_column_set_attributes (column, renderer,
-                                       "text", XFAE_MODEL_COLUMN_NAME,
+                                       "markup", XFAE_MODEL_COLUMN_NAME,
                                        NULL);
-  renderer = gtk_cell_renderer_text_new ();
+
   gtk_tree_view_append_column (GTK_TREE_VIEW (window->treeview), column);
 
   bbox = gtk_hbox_new (FALSE, 6);
@@ -179,11 +182,22 @@ xfae_window_init (XfaeWindow *window)
   button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
   g_signal_connect_swapped (G_OBJECT (button), "clicked",
                             G_CALLBACK (xfae_window_remove), window);
+  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
   g_signal_connect (G_OBJECT (window->selection), "changed",
                     G_CALLBACK (xfae_window_selection_changed), button);
   xfae_window_selection_changed (window->selection, button);
+
+  button = gtk_button_new_from_stock (GTK_STOCK_EDIT);
+  g_signal_connect_swapped (G_OBJECT (button), "clicked",
+                            G_CALLBACK (xfae_window_edit), window);
   gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
+
+  g_signal_connect (G_OBJECT (window->selection), "changed",
+                    G_CALLBACK (xfae_window_selection_changed), button);
+  xfae_window_selection_changed (window->selection, button);
 }
 
 
@@ -264,7 +278,7 @@ xfae_window_add (XfaeWindow *window)
   gchar        *descr;
   gchar        *command;
 
-  dialog = xfae_dialog_new ();
+  dialog = xfae_dialog_new (NULL, NULL, NULL);
   parent = gtk_widget_get_toplevel (GTK_WIDGET (window));
   gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
@@ -276,7 +290,7 @@ xfae_window_add (XfaeWindow *window)
       model = gtk_tree_view_get_model (GTK_TREE_VIEW (window->treeview));
       if (!xfae_model_add (XFAE_MODEL (model), name, descr, command, &error))
         {
-          xfce_err (error->message);
+          xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed adding \"%s\""), name);
           g_error_free (error);
         }
 
@@ -296,13 +310,32 @@ xfae_window_remove (XfaeWindow *window)
   GtkTreeModel     *model;
   GtkTreeIter       iter;
   GError           *error = NULL;
+  GtkWidget        *parent;
+  gchar            *name;
+  gboolean          remove_item;
+
+  parent = gtk_widget_get_toplevel (GTK_WIDGET (window));
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->treeview));
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
-      if (!xfae_model_remove (XFAE_MODEL (model), &iter, &error))
+      if (!xfae_model_get (XFAE_MODEL (model), &iter, &name, NULL, NULL, &error))
         {
-          xfce_err (error->message);
+          xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed to remove item"));
+          g_error_free (error);
+          return;
+        }
+
+      remove_item = xfce_dialog_confirm (GTK_WINDOW (parent), GTK_STOCK_REMOVE, NULL,
+                                         _("This will permanently remove the application "
+                                           "from the list of automatically started applications"),
+                                         _("Are you sure you want to remove \"%s\""), name);
+
+      g_free (name);
+
+      if (remove_item && !xfae_model_remove (XFAE_MODEL (model), &iter, &error))
+        {
+          xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed to remove item"));
           g_error_free (error);
         }
     }
@@ -310,6 +343,59 @@ xfae_window_remove (XfaeWindow *window)
 
 
 
+static void
+xfae_window_edit (XfaeWindow *window)
+{
+  GtkTreeSelection *selection;
+  GtkTreeModel     *model;
+  GtkTreeIter       iter;
+  GError           *error = NULL;
+  gchar            *name;
+  gchar            *descr;
+  gchar            *command;
+  GtkWidget        *parent;
+  GtkWidget        *dialog;
+
+  parent = gtk_widget_get_toplevel (GTK_WIDGET (window));
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->treeview));
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
+    {
+      if (!xfae_model_get (XFAE_MODEL (model), &iter, &name, &descr, &command, &error))
+        {
+          xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed to edit item"));
+          g_error_free (error);
+          return;
+        }
+
+      dialog = xfae_dialog_new (name, descr, command);
+      gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
+
+      g_free (command);
+      g_free (descr);
+      g_free (name);
+
+      if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
+        {
+	  gtk_widget_hide (dialog);
+
+	  xfae_dialog_get (XFAE_DIALOG (dialog), &name, &descr, &command);
+
+          if (!xfae_model_edit (XFAE_MODEL (model), &iter, name, descr, command, &error))
+            {
+              xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed to edit item \"%s\""), name);
+              g_error_free (error);
+            }
+
+          g_free (command);
+          g_free (descr);
+          g_free (name);
+        }
+      gtk_widget_destroy (dialog);
+    }
+}
+
+
 static void
 xfae_window_item_toggled (XfaeWindow *window,
                           gchar      *path_string)
@@ -325,7 +411,7 @@ xfae_window_item_toggled (XfaeWindow *window,
     {
       if (!xfae_model_toggle (XFAE_MODEL (model), &iter, &error))
         {
-          xfce_err (error->message);
+          xfce_dialog_show_error (NULL, error, _("Failed to toggle item"));
           g_error_free (error);
         }
     }
@@ -362,58 +448,3 @@ xfae_window_new (void)
 {
   return g_object_new (XFAE_TYPE_WINDOW, NULL);
 }
-
-
-
-
-#if 0
-/**
- * xfae_window_create_plug_child:
- *
- * Creates a widget that can be used to embed the window contents 
- * into a GtkPlug widget. After this function call, the XfaeWindow can
- * no longer be used and has to be re-created.
- *
- * Return value: A widget holding the most important contents of the 
- *               window.
- **/
-GtkWidget*
-xfae_window_create_plug_child (XfaeWindow *window)
-{
-  GtkWidget *vbox;
-  GtkWidget *bbox;
-  GtkWidget *button;
-
-  g_return_val_if_fail (XFAE_IS_WINDOW (window), NULL);
-
-  vbox = gtk_vbox_new (FALSE, 6);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
-  gtk_widget_show (vbox);
-
-  gtk_widget_reparent (window->ibox, vbox);
-  gtk_widget_show (window->ibox);
-
-  bbox = gtk_hbutton_box_new ();
-  gtk_box_set_spacing (GTK_BOX (bbox), 12);
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_CENTER);
-  gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, TRUE, 0);
-  gtk_widget_show (bbox);
-
-  button = gtk_button_new_from_stock (GTK_STOCK_ADD);
-  g_signal_connect_swapped (G_OBJECT (button), "clicked",
-                            G_CALLBACK (xfae_window_add), window);
-  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
-  gtk_widget_show (button);
-
-  button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
-  g_signal_connect_swapped (G_OBJECT (button), "clicked",
-                            G_CALLBACK (xfae_window_remove), window);
-  g_signal_connect (G_OBJECT (window->selection), "changed",
-                    G_CALLBACK (xfae_window_selection_changed), button);
-  xfae_window_selection_changed (window->selection, button);
-  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
-  gtk_widget_show (button);
-
-  return vbox;
-}
-#endif
